#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2006-12-12 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


def waterlevelEncode(params, validate=False):
	'''Create a waterlevel binary message payload to pack into an AIS Msg waterlevel.

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (field automatically set to "12")
	  - timetag_month(uint): Time the measurement represents  month 1..12
	  - timetag_day(uint): Time the measurement represents  day of the month 1..31
	  - timetag_hour(uint): Time the measurement represents  UTC hours 0..23
	  - timetag_min(uint): Time the measurement represents  minutes
	  - timetag_sec(uint): Time the measurement represents  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - stationloc_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - stationloc_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - sigma(float): Standard deviation of 1 second samples used to compute the water level height
	  - o(uint): Count of number of samples that fall outside a 3-sigma band about the mean
	  - levelinferred(bool): indicates that the water level value has been inferred
	  - flat_tolerance_exceeded(bool): flat tolerance limit was exceeded.  Need better descr
	  - rate_tolerance_exceeded(bool): rate of change tolerance limit was exceeded
	  - temp_tolerance_exceeded(bool): temperature difference tolerance limit was exceeded
	  - expected_height_exceeded(bool): either the maximum or minimum expected water level height limit was exceeded
	  - link_down(bool): Unable to communicate with the tide system.  All data invalid
	  - timeLastMeasured_month(uint): Time since last measured value was available  month 1..12
	  - timeLastMeasured_day(uint): Time since last measured value was available  day of the month 1..31
	  - timeLastMeasured_hour(uint): Time since last measured value was available  UTC hours 0..23
	  - timeLastMeasured_min(uint): Time since last measured value was available  minutes
	  - timeLastMeasured_sec(uint): Time since last measured value was available  seconds
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=366),16))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=1),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=12),12))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_month']),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_day']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_hour']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_min']),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_sec']),6))
	if 'stationid' in params:
		bvList.append(aisstring.encode(params['stationid'],42))
	else:
		bvList.append(aisstring.encode('@@@@@@@',42))
	if 'stationloc_longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['stationloc_longitude'])*Decimal('600000')),28))
	else:
		bvList.append(binary.bvFromSignedInt(108600000,28))
	if 'stationloc_latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['stationloc_latitude'])*Decimal('600000')),27))
	else:
		bvList.append(binary.bvFromSignedInt(54600000,27))
	if 'waterlevel' in params:
		bvList.append(binary.bvFromSignedInt(params['waterlevel'],16))
	else:
		bvList.append(binary.bvFromSignedInt(-32768,16))
	if 'datum' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['datum']),5))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=31),5))
	bvList.append(binary.float2bitvec(params['sigma']))
	if 'o' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['o']),8))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=255),8))
	if params["levelinferred"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["flat_tolerance_exceeded"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["rate_tolerance_exceeded"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["temp_tolerance_exceeded"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["expected_height_exceeded"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["link_down"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_month']),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_day']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_hour']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_min']),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_sec']),6))

	return binary.joinBV(bvList)

def waterlevelDecode(bv, validate=False):
	'''Unpack a waterlevel message 

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (field automatically set to "12")
	  - timetag_month(uint): Time the measurement represents  month 1..12
	  - timetag_day(uint): Time the measurement represents  day of the month 1..31
	  - timetag_hour(uint): Time the measurement represents  UTC hours 0..23
	  - timetag_min(uint): Time the measurement represents  minutes
	  - timetag_sec(uint): Time the measurement represents  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - stationloc_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - stationloc_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - sigma(float): Standard deviation of 1 second samples used to compute the water level height
	  - o(uint): Count of number of samples that fall outside a 3-sigma band about the mean
	  - levelinferred(bool): indicates that the water level value has been inferred
	  - flat_tolerance_exceeded(bool): flat tolerance limit was exceeded.  Need better descr
	  - rate_tolerance_exceeded(bool): rate of change tolerance limit was exceeded
	  - temp_tolerance_exceeded(bool): temperature difference tolerance limit was exceeded
	  - expected_height_exceeded(bool): either the maximum or minimum expected water level height limit was exceeded
	  - link_down(bool): Unable to communicate with the tide system.  All data invalid
	  - timeLastMeasured_month(uint): Time since last measured value was available  month 1..12
	  - timeLastMeasured_day(uint): Time since last measured value was available  day of the month 1..31
	  - timeLastMeasured_hour(uint): Time since last measured value was available  UTC hours 0..23
	  - timeLastMeasured_min(uint): Time since last measured value was available  minutes
	  - timeLastMeasured_sec(uint): Time since last measured value was available  seconds
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['dac']=366
	r['fid']=1
	r['efid']=12
	r['timetag_month']=int(bv[32:36])
	r['timetag_day']=int(bv[36:41])
	r['timetag_hour']=int(bv[41:46])
	r['timetag_min']=int(bv[46:52])
	r['timetag_sec']=int(bv[52:58])
	r['stationid']=aisstring.decode(bv[58:100])
	r['stationloc_longitude']=Decimal(binary.signedIntFromBV(bv[100:128]))/Decimal('600000')
	r['stationloc_latitude']=Decimal(binary.signedIntFromBV(bv[128:155]))/Decimal('600000')
	r['waterlevel']=binary.signedIntFromBV(bv[155:171])
	r['datum']=int(bv[171:176])
	r['sigma']=binary.bitvec2float(bv[176:208])
	r['o']=int(bv[208:216])
	r['levelinferred']=bool(int(bv[216:217]))
	r['flat_tolerance_exceeded']=bool(int(bv[217:218]))
	r['rate_tolerance_exceeded']=bool(int(bv[218:219]))
	r['temp_tolerance_exceeded']=bool(int(bv[219:220]))
	r['expected_height_exceeded']=bool(int(bv[220:221]))
	r['link_down']=bool(int(bv[221:222]))
	r['timeLastMeasured_month']=int(bv[222:226])
	r['timeLastMeasured_day']=int(bv[226:231])
	r['timeLastMeasured_hour']=int(bv[231:236])
	r['timeLastMeasured_min']=int(bv[236:242])
	r['timeLastMeasured_sec']=int(bv[242:248])
	return r

def waterlevelPrint(params, out=sys.stdout):
	'''Print a waterlevel message to stdout.

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (field automatically set to "12")
	  - timetag_month(uint): Time the measurement represents  month 1..12
	  - timetag_day(uint): Time the measurement represents  day of the month 1..31
	  - timetag_hour(uint): Time the measurement represents  UTC hours 0..23
	  - timetag_min(uint): Time the measurement represents  minutes
	  - timetag_sec(uint): Time the measurement represents  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - stationloc_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - stationloc_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - sigma(float): Standard deviation of 1 second samples used to compute the water level height
	  - o(uint): Count of number of samples that fall outside a 3-sigma band about the mean
	  - levelinferred(bool): indicates that the water level value has been inferred
	  - flat_tolerance_exceeded(bool): flat tolerance limit was exceeded.  Need better descr
	  - rate_tolerance_exceeded(bool): rate of change tolerance limit was exceeded
	  - temp_tolerance_exceeded(bool): temperature difference tolerance limit was exceeded
	  - expected_height_exceeded(bool): either the maximum or minimum expected water level height limit was exceeded
	  - link_down(bool): Unable to communicate with the tide system.  All data invalid
	  - timeLastMeasured_month(uint): Time since last measured value was available  month 1..12
	  - timeLastMeasured_day(uint): Time since last measured value was available  day of the month 1..31
	  - timeLastMeasured_hour(uint): Time since last measured value was available  UTC hours 0..23
	  - timeLastMeasured_min(uint): Time since last measured value was available  minutes
	  - timeLastMeasured_sec(uint): Time since last measured value was available  seconds
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	out.write("waterlevel:\n")
	out.write("	dac:                       "+str(+params['dac'])+"\n")
	out.write("	fid:                       "+str(+params['fid'])+"\n")
	out.write("	efid:                      "+str(+params['efid'])+"\n")
	out.write("	timetag_month:             "+str(+params['timetag_month'])+"\n")
	out.write("	timetag_day:               "+str(+params['timetag_day'])+"\n")
	out.write("	timetag_hour:              "+str(+params['timetag_hour'])+"\n")
	out.write("	timetag_min:               "+str(+params['timetag_min'])+"\n")
	out.write("	timetag_sec:               "+str(+params['timetag_sec'])+"\n")
	out.write("	stationloc_longitude:      "+str(+params['stationloc_longitude'])+"\n")
	out.write("	stationloc_latitude:       "+str(+params['stationloc_latitude'])+"\n")
	out.write("	waterlevel:                "+str(+params['waterlevel'])+"\n")
	out.write("	datum:                     "+str(+params['datum'])+"\n")
	out.write("	sigma:                     "+str(+params['sigma'])+"\n")
	out.write("	o:                         "+str(+params['o'])+"\n")
	out.write("	levelinferred:             "+str(+params['levelinferred'])+"\n")
	out.write("	flat_tolerance_exceeded:   "+str(+params['flat_tolerance_exceeded'])+"\n")
	out.write("	rate_tolerance_exceeded:   "+str(+params['rate_tolerance_exceeded'])+"\n")
	out.write("	temp_tolerance_exceeded:   "+str(+params['temp_tolerance_exceeded'])+"\n")
	out.write("	expected_height_exceeded:  "+str(+params['expected_height_exceeded'])+"\n")
	out.write("	link_down:                 "+str(+params['link_down'])+"\n")
	out.write("	timeLastMeasured_month:    "+str(+params['timeLastMeasured_month'])+"\n")
	out.write("	timeLastMeasured_day:      "+str(+params['timeLastMeasured_day'])+"\n")
	out.write("	timeLastMeasured_hour:     "+str(+params['timeLastMeasured_hour'])+"\n")
	out.write("	timeLastMeasured_min:      "+str(+params['timeLastMeasured_min'])+"\n")
	out.write("	timeLastMeasured_sec:      "+str(+params['timeLastMeasured_sec'])+"\n")

	return # Nothing to return



######################################################################
# UNIT TESTING
######################################################################
import unittest
def waterlevelTestParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['dac'] = 366
	params['fid'] = 1
	params['efid'] = 12
	params['timetag_month'] = 2
	params['timetag_day'] = 28
	params['timetag_hour'] = 23
	params['timetag_min'] = 45
	params['timetag_sec'] = 58
	params['stationid'] = 'A234567'
	params['stationloc_longitude'] = Decimal('-122.16328055555556')
	params['stationloc_latitude'] = Decimal('37.424458333333334')
	params['waterlevel'] = -97
	params['datum'] = 0
	params['sigma'] = -1.234
	params['o'] = 240
	params['levelinferred'] = False
	params['flat_tolerance_exceeded'] = True
	params['rate_tolerance_exceeded'] = False
	params['temp_tolerance_exceeded'] = False
	params['expected_height_exceeded'] = True
	params['link_down'] = False
	params['timeLastMeasured_month'] = 2
	params['timeLastMeasured_day'] = 28
	params['timeLastMeasured_hour'] = 23
	params['timeLastMeasured_min'] = 45
	params['timeLastMeasured_sec'] = 58

	return params

class Testwaterlevel(unittest.TestCase):
	'''Uses the testvalue tag text from each type to build a test case for the waterlevel message'''
	def testEncodeDecode(self):
	
		params = waterlevelTestParams()
		bits = waterlevelEncode(params)
		r = waterlevelDecode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['dac'],params['dac'])
		self.failUnlessEqual(r['fid'],params['fid'])
		self.failUnlessEqual(r['efid'],params['efid'])
		self.failUnlessEqual(r['timetag_month'],params['timetag_month'])
		self.failUnlessEqual(r['timetag_day'],params['timetag_day'])
		self.failUnlessEqual(r['timetag_hour'],params['timetag_hour'])
		self.failUnlessEqual(r['timetag_min'],params['timetag_min'])
		self.failUnlessEqual(r['timetag_sec'],params['timetag_sec'])
		self.failUnlessEqual(r['stationid'],params['stationid'])
		self.failUnlessAlmostEqual(r['stationloc_longitude'],params['stationloc_longitude'],5)
		self.failUnlessAlmostEqual(r['stationloc_latitude'],params['stationloc_latitude'],5)
		self.failUnlessEqual(r['waterlevel'],params['waterlevel'])
		self.failUnlessEqual(r['datum'],params['datum'])
		self.failUnlessAlmostEqual(r['sigma'],params['sigma'],7)
		self.failUnlessEqual(r['o'],params['o'])
		self.failUnlessEqual(r['levelinferred'],params['levelinferred'])
		self.failUnlessEqual(r['flat_tolerance_exceeded'],params['flat_tolerance_exceeded'])
		self.failUnlessEqual(r['rate_tolerance_exceeded'],params['rate_tolerance_exceeded'])
		self.failUnlessEqual(r['temp_tolerance_exceeded'],params['temp_tolerance_exceeded'])
		self.failUnlessEqual(r['expected_height_exceeded'],params['expected_height_exceeded'])
		self.failUnlessEqual(r['link_down'],params['link_down'])
		self.failUnlessEqual(r['timeLastMeasured_month'],params['timeLastMeasured_month'])
		self.failUnlessEqual(r['timeLastMeasured_day'],params['timeLastMeasured_day'])
		self.failUnlessEqual(r['timeLastMeasured_hour'],params['timeLastMeasured_hour'])
		self.failUnlessEqual(r['timeLastMeasured_min'],params['timeLastMeasured_min'])
		self.failUnlessEqual(r['timeLastMeasured_sec'],params['timeLastMeasured_sec'])

############################################################
if __name__=='__main__':

    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)

    #sys.exit('EARLY EXIT - FIX: remove')
    myparser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
                        help='run the documentation tests')
    myparser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
                        help='run the unit tests')
    myparser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
                        help='Make the test output verbose')

    (options,args) = myparser.parse_args()
    success=True

    if options.doctest:
	import os; print os.path.basename(sys.argv[0]), 'doctests ...',
	sys.argv= [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	import doctest
	numfail,numtests=doctest.testmod()
	if numfail==0: print 'ok'
	else: 
	    print 'FAILED'
	    success=False

    if not success:
	sys.exit('Something Failed')

    del success # Hide success from epydoc

    if options.unittest:
	sys.argv = [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	unittest.main()
